import collections
import unittest

from sate import types


class TestSeparateDeps(unittest.TestCase):
    def check(self, src, expected):
        self.assertEqual(types.separate_deps_from_directives(src), expected)

    def test_empty(self):
        self.check([], ([], []))

    def test_just_deps(self):
        self.check([types.Call('deps', ['a'])], (['a'], []))

    def test_both(self):
        self.check([types.Call('deps', ['a']),
                    types.Call('b')], (['a'], [types.Call('b')]))


class TestMakeTargetGraph(unittest.TestCase):
    def setUp(self):
        self.targets = collections.OrderedDict([
            ('a', types.Target('a').with_deps('b c d')),
            ('b', types.Target('b').with_deps('c')),
            ('c', types.Target('c').with_deps('d')),
            ('d', types.Target('d')),
        ])

    def test_from_b(self):
        graph = types.make_target_graph(self.targets, 'b')
        self.assertEqual(graph, {
            'b': set(['c']),
            'c': set(['d']),
            'd': set(),
        })

    def test_from_d(self):
        graph = types.make_target_graph(self.targets, 'd')
        self.assertEqual(graph, {
            'd': set(),
        })


class TestDeps(unittest.TestCase):
    def test_no_deps(self):
        self.assertEqual(types.Target('a').deps(), [])

    def test_one_dep(self):
        target = types.Target('a', directives=[types.Call('deps', ['b'])])
        self.assertEqual(target.deps(), ['b'])

    def test_two_deps(self):
        target = types.Target('a', directives=[types.Call('deps', ['b', 'c'])])
        self.assertEqual(target.deps(), ['b', 'c'])

    def test_dag(self):
        t_a = types.Target('a', directives=[types.Call('deps', ['b', 'd'])])
        t_b = types.Target('b', directives=[types.Call('deps', ['c', 'd'])])
        t_c = types.Target('c')
        t_d = types.Target('d')
        satefile = types.Satefile((t_a, t_b, t_c, t_d))
        self.assertEqual(satefile.run_order('a'), ['c', 'd', 'b', 'a'])

    def test_four_targets(self):
        satefile = types.Satefile([
            types.Target('lint'),
            types.Target('format'),
            types.Target('test'),
            types.Target('sanity').with_deps('format', 'lint', 'test')
        ])
        self.assertEqual(
            satefile.run_order('sanity'), ['format', 'lint', 'test', 'sanity'])



import io
import unittest

from sate.parse import compose, load_satefile, parse_line
from sate.types import Call, Command, Comment, Satefile, Target


class TestParseLine(unittest.TestCase):
    def check(self, src, expected):
        self.assertEqual(list(parse_line(src)), expected)

    def test_command(self):
        self.check('abc', [Command('abc')])

    def test_comment(self):
        self.check('# abc', [Comment(' abc')])

    def test_target(self):
        self.check('[a]', [Target('a')])

    def test_target_with_comment(self):
        self.check('[a] #b', [Target('a'), Comment('b')])

    def test_directive(self):
        self.check('[a] b', [Command(directives=[Call('a')], text='b')])


class TestCompose(unittest.TestCase):
    def check(self, src, expected):
        self.assertEqual(list(compose(src)), expected)

    def test_empty(self):
        self.check([], [])

    def test_empty_target(self):
        self.check([Target('a')], [Target('a')])

    def test_simple_target(self):
        self.check([Target('a'), Command('b')], [Target('a', [Command('b')])])


class TestFullFile(unittest.TestCase):
    def check(self, src, expected):
        rfile = io.StringIO(src)
        self.assertEqual(load_satefile(rfile), expected)

    def test_empty(self):
        self.check('', Satefile())

    # def test_complex(self):
    #     self.check('''
    #     # hello
    #     [a] # world
    #     # hello
    #     b # world

    #     [c deps(a)]
    #     [d] e
    #     ''', Satefile([Target('a', [Command('b')]),
    #                    Target('c', [Command('e', Directive('d'))],
    #                           Directive('deps', ['a']))]))



import unittest

from sate import rules, types


class TestCall(unittest.TestCase):
    def test_one_char(self):
        self.assertEqual(rules.Call.parse('a()'), types.Call('a'))

    def test_two_char(self):
        self.assertEqual(rules.Call.parse('ab()'), types.Call('ab'))

    def test_no_parens(self):
        self.assertEqual(rules.Call.parse('a'), types.Call('a'))

    def test_two_char_no_parens(self):
        self.assertEqual(rules.Call.parse('ab'), types.Call('ab'))

    def test_args(self):
        self.assertEqual(
            rules.Call.parse('a(b c d)'), types.Call('a', ['b', 'c', 'd']))


class TestCommandTag(unittest.TestCase):
    def test_one_char(self):
        self.assertEqual(rules.CommandTag.parse('[a]'), [types.Call('a')])

    def test_two_char(self):
        self.assertEqual(rules.CommandTag.parse('[ab]'), [types.Call('ab')])

    def test_call(self):
        self.assertEqual(rules.CommandTag.parse('[a()]'), [types.Call('a')])


class TestTargetTag(unittest.TestCase):
    def test_one_char(self):
        self.assertEqual(rules.TargetTag.parse('[a]'), types.Target('a'))

    def test_two_char(self):
        self.assertEqual(rules.TargetTag.parse('[ab]'), types.Target('ab'))

    def test_no_args(self):
        self.assertEqual(
            rules.TargetTag.parse('[a b]'),
            types.Target('a', directives=[types.Call('b')]))
